/*
 * 29.08.2004 by Andreas Grunewald
 */
package org.jd3lib;

/**
 * Frames that allow different types of text encoding contains a text encoding
 * description byte. Possible encodings:
 * 
 * $00 ISO-8859-1 [ISO-8859-1]. Terminated with $00.
 * 
 * $01 UTF-16 [UTF-16] encoded Unicode [UNICODE] with BOM. All strings in the
 * same frame SHALL have the same byteorder. Terminated with $00 00.
 * 
 * $02 UTF-16BE [UTF-16] encoded Unicode [UNICODE] without BOM. Terminated with
 * $00 00.
 * 
 * $03 UTF-8 [UTF-8] encoded Unicode [UNICODE]. Terminated with $00.
 * 
 * Encoding Information is taken from Id3v2.4.0
 * 
 * @author Andreas Grunewald
 *  
 */
public class FieldEncoding
        extends Id3Field {

    public static final byte ISO88591 = 0x00;

    public static final byte UTF16 = 0x01;

    public static final byte UTF16BE = 0x02;

    public static final byte UTF8 = 0x03;

    private byte encoding;

    /**
     * @param encoding
     */
    public FieldEncoding(byte enc) {
        if (enc >= 0x00 && enc <= 0x03)
            this.encoding = enc;
        else
            encoding = ISO88591;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.gruni.thejee.Id3Field#getBytes()
     */
    public byte[] getBytes() {
        return new byte[] { encoding };
    }

    public void setField(byte[] inType) {
        setField(inType[0]);
    }

    public void setField(Byte inByte) {
        setField(inByte.byteValue());
    }

    /**
     * @param b
     */
    public void setField(byte b) {
        encoding = b;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.gruni.thejee.Id3Field#getValue()
     */
    public Object getValue() {
        return new Byte(encoding);
    }

    public String toString() {
        String out;
        switch (encoding) {
            case ISO88591:
                out = "ISO-8859-1";
                break;
            case UTF16:
                out = "UTF-16";
                break;
            case UTF16BE:
                out = "UTF-16BE";
                break;
            case UTF8:
                out = "UTF-8";
                break;
            default:
                out = "unknown";
        }
        return out;
    }

}