/*
 * $Id: Id3FrameCOMM.java,v 1.10 2005/04/27 08:37:44 gruni Exp $
 * $Author: gruni $ 
 * 
 * $Log: Id3FrameCOMM.java,v $
 * Revision 1.10  2005/04/27 08:37:44  gruni
 * changes due to refactoring
 *
 */
package org.jd3lib;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;

import org.jd3lib.util.DebugOut;

/**
 * 4.11. Comments
 * 
 * This frame is indended for any kind of full text information that does not
 * fit in any other frame. It consists of a frame header followed by encoding,
 * language and content descriptors and is ended with the actual comment as a
 * text string. Newline characters are allowed in the comment text string. There
 * may be more than one comment frame in each tag, but only one with the same
 * language and content descriptor.
 * 
 * <Header for 'Comment', ID: "COMM">
 * 
 * Text encoding $xx
 * 
 * Language $xx xx xx
 * 
 * Short content descrip. <text string according to encoding> $00 (00)
 * 
 * The actual text <full text string according to encoding>
 * 
 * @author Andreas Grunewald
 */
public class Id3FrameCOMM
    extends Id3FrameMultiple {

  /**
   * @param frameData
   * @param header
   */
  public Id3FrameCOMM(ByteBuffer frameData, Id3v2FrameHeader header) {
    super(frameData, header);
    // Later 0.7 Auto-generated constructor stub
  }

  FieldEncoding TextEncoding;

  FieldLanguage Language;

  FieldTerminatedString ShortContentDescrip;

  String TheActualText;

  /*
   * (non-Javadoc)
   * 
   * @see de.gruni.thejee.Id3Frame#initialize(java.io.InputStream,
   *      de.gruni.thejee.Id3v2FrameHeader)
   */
  protected void initializeExisting(InputStream tagData,
      Id3v2FrameHeader header2) {
    DebugOut.println("RunningCOMM initialize");
    byte[] temp = new byte[header2.getSize()];
    try {
      tagData.read(temp);
    } catch (IOException e) {
      //LATER 1.5 Auto-generated catch block
      e.printStackTrace();
    }
    ByteArrayInputStream inStream = new ByteArrayInputStream(temp);
    /*
     * TODO Load data from Temp Array into fields
     * 
     * Text encoding $xx
     * 
     * Language $xx xx xx
     * 
     * Short content descrip. <text string according to encoding> $00 (00)
     * 
     * The actual text <full text string according to encoding>
     */
    //DONEXT Fill shortcontent Description
    TextEncoding = new FieldEncoding((byte) inStream.read());
    if (inStream.available() >= 4)
      Language = new FieldLanguage((byte) inStream.read(), (byte) inStream
          .read(), (byte) inStream.read());
    else
      Language = new FieldLanguage();
    ShortContentDescrip = new FieldTerminatedString(FieldTerminatedString
        .readUntilZero(inStream));
    if (inStream.available() >= 1) {
      byte[] text = new byte[inStream.available()];
      try {
        inStream.read(text);
      } catch (IOException e1) {
        e1.printStackTrace();
      }
      TheActualText = new String(text);
    } else
      TheActualText = "";

    //data = new FrameData(temp);
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.gruni.thejee.Id3Frame#getDataString()
   */
  protected String getFrameString() {
    return "Text encoding: " + TextEncoding + "\nLanguage: " + Language
        + "\nShort content descrip.: " + ShortContentDescrip
        + "\nThe actual text: " + TheActualText;
  }

  public void setComment(String comment) {
    TheActualText = comment;
  }

  public String getComment() {
    return TheActualText;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3Frame#initializeNew(org.jd3lib.Id3v2FrameHeader)
   */
  protected void initializeNew(Id3v2FrameHeader header) {
    TextEncoding = new FieldEncoding(FieldEncoding.ISO88591);
    Language = new FieldLanguage((byte) 0, (byte) 0, (byte) 0);
    ShortContentDescrip = new FieldTerminatedString();
    TheActualText = "";

  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3Frame#getFrameData()
   */
  protected ByteArrayOutputStream getFrameData() {
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    try {
      out.write(TextEncoding.getBytes());//Text encoding $xx

      out.write(Language.getBytes());// Language $xx xx xx
      out.write(ShortContentDescrip.getBytes());//Short content descrip.
      // <text string according
      // to encoding> $00 (00)
      out.write(TheActualText.getBytes());//The actual text <full text
      // string according to encoding>
    } catch (IOException e) {
      //LATER 1.5 Auto-generated catch block
      e.printStackTrace();
    }
    return out;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3FrameMultiple#isDifferent(org.jd3lib.Id3FrameMultiple)
   */
  public boolean isDifferent(Id3FrameMultiple toCompare) {
    // LATER 0.1 Auto generated method stub
    return false;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3Frame#initializeExisting(java.nio.ByteBuffer,
   *      org.jd3lib.Id3v2FrameHeader)
   */
  protected void initializeExisting(ByteBuffer frameData,
      Id3v2FrameHeader header) {
    DebugOut.println("RunningCOMM initialize");
    ByteArrayInputStream inStream = new ByteArrayInputStream(frameData.array());
    /*
     * TODO Load data from Temp Array into fields
     * 
     * Text encoding $xx
     * 
     * Language $xx xx xx
     * 
     * Short content descrip. <text string according to encoding> $00 (00)
     * 
     * The actual text <full text string according to encoding>
     */
    //DONEXT Fill shortcontent Description
    TextEncoding = new FieldEncoding((byte) inStream.read());
    if (inStream.available() >= 4)
      Language = new FieldLanguage((byte) inStream.read(), (byte) inStream
          .read(), (byte) inStream.read());
    else
      Language = new FieldLanguage();
    ShortContentDescrip = new FieldTerminatedString(FieldTerminatedString
        .readUntilZero(inStream));
    if (inStream.available() >= 1) {
      byte[] text = new byte[inStream.available()];
      try {
        inStream.read(text);
      } catch (IOException e1) {
        e1.printStackTrace();
      }
      TheActualText = new String(text);
    } else
      TheActualText = "";

    //data = new FrameData(temp);

  }

}