/*
 * $Id: Id3FrameWXXX.java,v 1.9 2005/04/27 08:47:29 gruni Exp $
 * $Author: gruni $ 
 * 
 * $Log: Id3FrameWXXX.java,v $
 * Revision 1.9  2005/04/27 08:47:29  gruni
 * -problem with class in GCJ noted
 *
 */
package org.jd3lib;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.ByteBuffer;

import org.jd3lib.compatible.Logger;
import org.jd3lib.util.DebugOut;

/**
 * 4.3.2. User defined URL link frame
 * 
 * This frame is intended for URL [URL] links concerning the audiofile in a
 * similar way to the other "W"-frames. The frame body consists of a description
 * of the string, represented as a terminated string, followed by the actual
 * URL. The URL is always encoded with ISO-8859-1 [ISO-8859-1]. There may be
 * more than one "WXXX" frame in each tag, but only one with the same
 * description.
 * 
 * <Header for 'User defined URL link frame', ID: "WXXX">
 * 
 * Text encoding $xx
 * 
 * Description <text string according to encoding> $00 (00)
 * 
 * URL <text string>
 * 
 * @author Andreas Grunewald
 *  
 */
public class Id3FrameWXXX
    extends Id3FrameMultiple {

  /**
   * @param frameData
   * @param header
   */
  public Id3FrameWXXX(ByteBuffer frameData, Id3v2FrameHeader header) {
    super(frameData, header);
    // LATER 0.7 Auto-generated constructor stub
  }

  //DONEXT put getter and setter
  FieldEncoding TextEncoding;

  String Description;

  URI URLdata;

  /*
   * (non-Javadoc)
   * 
   * @see de.gruni.thejee.Id3Frame#initialize(java.io.InputStream,
   *      de.gruni.thejee.Id3v2FrameHeader)
   */
  protected void initializeExisting(InputStream tagData,
      Id3v2FrameHeader header2) {
    DebugOut.println("RunningWXXX initialize");
    byte[] temp = new byte[header2.getSize()];
    try {
      tagData.read(temp);
    } catch (IOException e) {
      //LATER 1.5 Auto-generated catch block
      e.printStackTrace();
    }
    //LATER 0.8 Remove this debugging code
    //for (int i = 0; i < temp.length; i++) {
    //            System.out.print("[" + temp[i] + "]");
    //        }
    //        System.out.println("\n");
    //System.exit(0);
    TextEncoding = new FieldEncoding(temp[0]);
    if (temp.length > 1) {
      ByteArrayOutputStream desStream = new ByteArrayOutputStream();
      for (int i = 1; i < temp.length; i++) {
        if (temp[i] != 0x00)
          desStream.write(temp[i]);
        else
          break;
      }
      String URLstr = new String(temp, desStream.size() + 2, temp.length
          - (desStream.size() + 2));
      // System.out.println("["+URLstr+"]");
      try {
        URLdata = new URI(URLstr);
        Description = desStream.toString();
      } catch (URISyntaxException e1) {
        URLdata = null;
        Description = desStream.toString()
            + new String(temp, desStream.size() + 2, temp.length
                - (desStream.size() + 2));
      }
    } else {
      Description = new String();
      URLdata = null;
    }

  }

  //TODO By output to bytes if URL is NULL put Description into URL field.

  /*
   * (non-Javadoc)
   * 
   * @see de.gruni.thejee.Id3Frame#getDataString()
   */
  protected String getFrameString() {
    return "Text encoding: " + TextEncoding + "\nDescription: " + Description
        + "\nURL: " + URLdata;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3Frame#initializeNew(org.jd3lib.Id3v2FrameHeader)
   */
  protected void initializeNew(Id3v2FrameHeader header) {
    TextEncoding = new FieldEncoding(FieldEncoding.ISO88591);
    Description = new String();
    URLdata = null;

  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3Frame#getFrameData()
   */
  protected ByteArrayOutputStream getFrameData() {
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    try {
      out.write(TextEncoding.getBytes());//Text encoding $xx

      if (URLdata == null) {
        out.write(0);//Terminated Null Descriptor
        out.write(Description.getBytes());//Write Description as URL
      } else {
        out.write(Description.getBytes());
        out.write(0);//termination of Description
        out.write(URLdata.toString().getBytes());// URL <text string>
      }

    } catch (IOException e) {
      // LATER 1.5 Auto-generated catch block
      e.printStackTrace();
    }

    return out;
  }

  public String getDescription() {
    return Description;
  }

  public void setDescription(String description) {
    this.Description = description;
  }

  public FieldEncoding getTextEncoding() {
    return TextEncoding;
  }

  public void setTextEncoding(FieldEncoding textEncoding) {
    TextEncoding = textEncoding;
  }

  public URI getURL() {
    return URLdata;
  }

  public void setURL(String url) {
    try {
      URLdata = new URI(url);
    } catch (URISyntaxException e) {
      URLdata = null;
      Description = url;
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3FrameMultiple#isDifferent(org.jd3lib.Id3FrameMultiple)
   */
  public boolean isDifferent(Id3FrameMultiple toCompare) {
    // LATER 0.1 Auto generated method stub
    return false;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3Frame#initializeExisting(java.nio.ByteBuffer,
   *      org.jd3lib.Id3v2FrameHeader)
   */
  protected void initializeExisting(ByteBuffer frameData,
      Id3v2FrameHeader header) {
    Logger.global.fine("RunningWXXX initialize");
    byte[] temp = frameData.array();
    //        for (int i = 0; i < temp.length; i++) {
    //            System.out.print("[" + temp[i] + "]");
    //        }
    //        System.out.println("\n");
    //System.exit(0);
    TextEncoding = new FieldEncoding(temp[0]);
    if (temp.length > 1) {
      ByteArrayOutputStream desStream = new ByteArrayOutputStream();
      for (int i = 1; i < temp.length; i++) {
        if (temp[i] != 0x00)
          desStream.write(temp[i]);
        else
          break;
      }
      String URLstr = new String(temp, desStream.size() + 2, temp.length
          - (desStream.size() + 2));
      // System.out.println("["+URLstr+"]");
      try {
        //TODO GCJ BUG this Constructor desn't work thus this class needs to be
        // excluded from GCJ distribution
        URLdata = new URI(URLstr);
        Description = desStream.toString();
      } catch (URISyntaxException e1) {
        URLdata = null;
        Description = desStream.toString()
            + new String(temp, desStream.size() + 2, temp.length
                - (desStream.size() + 2));
      }
    } else {
      Description = new String();
      URLdata = null;
    }

  }
}