/*
 * $Id: Id3v2TagFactory.java,v 1.1 2005/04/27 09:02:50 gruni Exp $
 * $Author: gruni $ 
 * $Revision: 1.1 $
 * $Date: 2005/04/27 09:02:50 $
 * $Log: Id3v2TagFactory.java,v $
 * Revision 1.1  2005/04/27 09:02:50  gruni
 * classes for ID3v2.2
 *
 */
package org.jd3lib;

import java.lang.reflect.Constructor;
import java.nio.ByteBuffer;

import org.jd3lib.compatible.Logger;

/**
 * @author Grunewald
 */
public abstract class Id3v2TagFactory {

  Id3v2TagHeader header;

  /**
   * @deprecated I don't think I need this.
   */
  protected Id3v2TagFactory(Id3v2TagHeader nextHeader) {
    header = nextHeader;
  }

  /**
   * @param b
   * @param versionMinor
   * @param flags
   * @param size
   */
  public Id3v2TagFactory(byte b, byte versionMinor, byte flags, int size) {
    header = new Id3v2TagHeader(b, versionMinor, flags, size);
  }

  /**
   * @return
   */
  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3v2TagFactory#getHeader()
   */
  public Id3v2TagHeader getHeader() {
    return header;
  }

  /**
   * @param tag
   * @return
   */
  public Id3FrameList getFrames(ByteBuffer tag) throws InstantiationException {
    if (header != null) {
      //TODO clean long avail = 0;
      Id3FrameList frames = null;
      try {
        frames = process(tag);
      } catch (Exception e) {
        handelInitException(e);
      }
      return frames;
    }
    throw new InstantiationException("Invalid Header can not create Tag");

  }

  protected abstract Id3FrameList process(ByteBuffer tag);

  /**
   * @param frame
   * @param head
   */
  protected Id3Frame createFrame(ByteBuffer frame, Id3v2FrameHeader head) {
    //TODO remove Logger.global.fine("STARTING Create Frame for
    // header:\n"+head);
    Id3Frame newFrame = null;
    try {
      Class team = Class.forName(Id3Frame.class.getName() + head.getFrameID());
      Class[] argClass = { ByteBuffer.class, Id3v2FrameHeader.class };
      Object[] args = { frame, head };
      Constructor constT = team.getConstructor(argClass);
      newFrame = (Id3Frame) constT.newInstance(args);
    } catch (ClassNotFoundException e) {
      //If any unknown Tag occures create a ID3FrameUnknown for that
      // containing
      // simpl binary data.
      Logger.global
          .info("Please Report this Frame "
              + e.getMessage()
              + " as a feature Request to the jd3lib homepage\n Unknown frame Will be created.");
      newFrame = new Id3FrameUnknown(frame, head);

    } catch (Exception e) {
      handelInitException(e);
      //TODO findout what this is faulty = true;
    }
    return newFrame;
  }

  /**
   * @param e
   */
  public static void handelInitException(Exception e) {
    if (e instanceof ClassNotFoundException) {
      System.err.println("Please Report this Frame " + e.getMessage()
          + " as a feature Request to the jd3lib homepage");
      //e.printStackTrace();
      //faulty = true;
    } else
      e.printStackTrace();
  }

}