/*
 * $Id: Header.java,v 1.4 2005/03/02 08:10:49 gruni Exp $
 * $Author: gruni $ 
 * $Revision: 1.4 $
 * $Date: 2005/03/02 08:10:49 $
 * $Log: Header.java,v $
 * Revision 1.4  2005/03/02 08:10:49  gruni
 * a lot of structural refactoring
 *
 * Revision 1.3  2005/01/27 17:29:18  gruni
 * updated comments and corrected naming of Finals
 *
 * Revision 1.8  2004/12/16 16:59:52  gruni
 * a lot of updates switching to NIO and adding ID3v1
 */
package org.jd3lib.archoslib;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

/**
 * @author Grunewald
 * 
 * 
 * 1. Header
 * 
 * struct hdr_str {
 * 
 * uint8_t magic[4]; = "JBML"
 * 
 * uint32_t version; = 0x00000101
 * 
 * uint32_t num_files;
 * 
 * uint32_t num_lists;
 * 
 * uint32_t offset_files;
 * 
 * uint32_t offset_lists;
 * 
 * uint32_t offset_list_entries;
 * 
 * uint32_t offset_paths;
 * 
 * uint32_t offset_strings;
 * 
 * uint32_t offset_private_data;
 * 
 * uint32_t search_list;
 * 
 * uint8_t res[468]; } num_files, num_lists is the number of files and lists
 * respectively. The sum of these two must be less than 65536.
 * 
 * 
 * 
 * 
 *  
 */
public class Header
        extends Section {
    private static final byte magic[] = { 'J', 'B', 'M', 'L' };

    /*
     * The used Value for version was reverse engineered from a Musicmatch
     * Jukebox file the Standard is wrong with stating that version needs to be
     * version = 0x00000101 which was defined in the 1.5 Lib standard
     */
    private static final int VERSION = 0x00000102;

    private static final int MAX_NUMBER = 65536;

    /**
     * num_files, num_lists is the number of files and lists respectively. The
     * sum of these two must be less than 65536.
     */
    private int num_files;

    /**
     * num_files, num_lists is the number of files and lists respectively. The
     * sum of these two must be less than 65536.
     */
    private int num_lists;

    /**
     * offset_files, and offset_lists is the offset in bytes from the start of
     * the file to the respective structure holding information about files and
     * lists.
     */
    private int offset_files;

    /**
     * offset_files, and offset_lists is the offset in bytes from the start of
     * the file to the respective structure holding information about files and
     * lists.
     */
    private int offset_lists;

    /**
     * offset_list_entries is the offset in bytes from the start of the file to
     * the variable length structure holding the individual list entries.
     */
    private int offset_list_entries;

    /**
     * offset_paths is the offset in bytes from the start of the file to the
     * variable length structures which hold the paths to the files.
     */
    private int offset_paths;

    /**
     * offset_strings is the offset in bytes from the start of the file to the
     * string pool, where all text strings used in files, lists and paths
     * stored.
     */
    private int offset_strings;

    /**
     * offset_private_data is the offset in bytes from the start of the file to
     * additional private data. The contents of the data section are not
     * specified and will not be used by Archos multimedia products. Private
     * data must be located at the end of the library file. If there is no
     * private data present, the offset should equal the file size and therefore
     * point to the end of the file.
     */
    private int offset_private_data;

    /**
     * search_list gives the number of the list which will be used for searching
     * through all content.
     */
    private int search_list;

    /**
     * res fills the header structure to 512 bytes total length (for future
     * expansion).
     */
    private static final byte RES[] = new byte[468];

    /*
     * (non-Javadoc)
     * 
     * @see org.jd3lib.archoslib.Section#toByteBuffer()
     */
    protected ByteBuffer toByteBuffer() {
        ByteBuffer rest = ByteBuffer.allocate(512);
        rest.order(ByteOrder.LITTLE_ENDIAN);
        rest.put(magic);
        rest.putInt(VERSION);
        rest.putInt(num_files);
        rest.putInt(num_lists);
        rest.putInt(offset_files);
        rest.putInt(offset_lists);
        rest.putInt(offset_list_entries);
        rest.putInt(offset_paths);
        rest.putInt(offset_strings);
        rest.putInt(offset_private_data);
        rest.putInt(search_list);
        rest.put(RES);
        rest.flip();
        return rest;
    }

    protected void setNum_files(int in_num_files) {
        this.num_files = in_num_files;
        checkMaximum();
    }

    /**
     * Checks if values are in the Range
     */
    private void checkMaximum() {
        if (num_files + num_lists >= MAX_NUMBER)
                throw new RuntimeException(
                        "To many Files and Lists the sum of Files and Lists has to be <="
                                + MAX_NUMBER);

    }

    protected void setNum_lists(int in_num_lists) {
        this.num_lists = in_num_lists;
        checkMaximum();
    }

    protected void setOffset_files(int in_offset_files) {
        this.offset_files = in_offset_files;
    }

    protected void setOffset_list_entries(int offset_list_entries_in) {
        this.offset_list_entries = offset_list_entries_in;
    }

    protected void setOffset_lists(int offset_lists_in) {
        this.offset_lists = offset_lists_in;
    }

    protected void setOffset_paths(int offset_paths_in) {
        this.offset_paths = offset_paths_in;
    }

    protected void setOffset_private_data(int offset_private_data_in) {
        this.offset_private_data = offset_private_data_in;
    }

    protected void setOffset_strings(int offset_strings_in) {
        this.offset_strings = offset_strings_in;
    }

    protected void setSearch_list(int search_list_in) {
        this.search_list = search_list_in;
    }
}