/*
 * $Id: LibraryWriter.java,v 1.2 2005/04/27 08:02:19 gruni Exp $
 * $Author: gruni $ 
 * $Revision: 1.2 $
 * $Date: 2005/04/27 08:02:19 $
 * $Log: LibraryWriter.java,v $
 * Revision 1.2  2005/04/27 08:02:19  gruni
 * Minor changes, new Logging framework used
 *
 * Revision 1.1  2005/04/04 15:19:42  gruni
 * the processor
 *
 */
package org.jd3lib.archoslib;

//DONEXT Check if all BitOrders are Little Endian
//DONEXT Care and check if Unicode UTF-8 is handeled correctly eg ""
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;
import java.util.Iterator;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.EventListenerList;

import org.jd3lib.compatible.Logger;
import org.jd3lib.compatible.Queue;

/**
 * @author Grunewald
 */
public class LibraryWriter implements Runnable {
  public static final String FILE_NAME = "lib.jbm";

  ChangeEvent changeEvent = null;

  Files files;

  EventListenerList listenerList = new EventListenerList();

  ListEntries listentries;

  Lists lists;

  Paths paths;

  private File root;

  Strings strings;

  private int percentage;

  public Header header;

  private int fileCount = 0;

  private int processed = 0;

  private Queue stringQ;

  private Queue foundFiles;

  private boolean backup;

  private boolean ready;

  private boolean cancel;

  /**
   * @param file
   */
  public LibraryWriter(File workingDirectory) {
    init(workingDirectory);
  }

  /**
   * 
   */
  public LibraryWriter() {
    ready = false;
  }

  public void init(File workingDirectory) {
    ready = true;
    //  TODO v2.0 make sure root is Directory
    root = workingDirectory; //root has to be assigned first because
    // it's final
    initializeClassVariables();
  }

  /**
   * @param makeBackup
   * @param queue
   * Iterates over the Found MP3s and inserts the MetaData into the Strings and
   *  
   */
  public void stepTwo(Queue inStrings, Queue inFound, boolean makeBackup) {
    if (ready) {
      // System.out.println("Ready checking vars");
      // if (inStrings != null) System.out.println("instring is OK");
      // if (inFound!= null) System.out.println("inFound is OK");      
      stringQ = inStrings;
      foundFiles = inFound;
      backup = makeBackup;
      run();
    } else
      throw new RuntimeException(
          "Library not initialized please call init() first");
  }

  /**
   * @param strings2
   */
  private void addStrings(Queue strings2) {
    while (!strings2.isEmpty()) {
      strings.put((String) strings2.remove());
    }
  }

  /**
   * @return
   */
  private int getWorkingDirLength() {

    try {
      String myPath = root.getCanonicalPath();
      //System.out.println("ROOTPATH:" + myPath);
      return myPath.length();
    } catch (IOException e) {
      e.printStackTrace();
      return 0;
    }
  }

  private void initializeClassVariables() {
    header = new Header();
    listentries = new ListEntries();
    strings = new Strings();
    paths = new Paths(getWorkingDirLength(), this.strings);
    files = new Files(this.strings, this.paths);
    lists = new Lists(this.strings);

    //TODO v1.0 add private data fingerprint.
    //privateData = new PrivateData();
  }

  public int getFileCount() {
    return fileCount;
  }

  public int getProcessed() {
    return processed;
  }

  private void setPercentage(int value) {
    percentage = value;
    fireStateChanged();
  }

  // Notify all listeners that have registered interest for
  // notification on this event type.  The event instance 
  // is lazily created using the parameters passed into 
  // the fire method.

  protected void fireStateChanged() {
    //  Guaranteed to return a non-null array
    Object[] listeners = listenerList.getListenerList();
    // Process the listeners last to first, notifying
    // those that are interested in this event
    /* TODO Reimplement original Code when GCJ updated
     * Original Code from the API had to be modified because of the GCJ
     for (int i = listeners.length-2; i>=0; i-=2) {
     Logger.global.info("Inside for loop for Statechange");
     if (listeners[i]==ChangeListener.class) {
     // Lazily create the event:
     if (changeEvent == null)
     changeEvent = new ChangeEvent(this);
     Logger.global.info("FIRE STATECHANGE");
     ((ChangeListener)listeners[i+1]).stateChanged(changeEvent);
     
     }
     }*/
    for (int i = listeners.length - 1; i >= 0; i--) {
      if (listeners[i] instanceof ChangeListener) {
        // Lazily create the event:
        if (changeEvent == null) changeEvent = new ChangeEvent(this);
        ((ChangeListener) listeners[i]).stateChanged(changeEvent);

      }
    }
  }

  public void addChangeListener(ChangeListener l) {
    listenerList.add(ChangeListener.class, l);
  }

  /**
   * @return
   */
  public int getPercentage() {
    return percentage;
  }

  public void removeChangeListener(ChangeListener l) {
    listenerList.remove(ChangeListener.class, l);
  }

  /**
   *  
   */
  public void write() {
    if (backup) makeBackup();
    FileOutputStream outStream;
    try {
      outStream = new FileOutputStream(root.getCanonicalPath() + "/"
          + FILE_NAME);

      FileChannel out = outStream.getChannel();

      out.position(512);
      //Header is written last because filepositions have to be known
      //  System.out.println("Currentpos:" + out.position()
      //         + " skipped first 512 because the header is written last");//1
      header.setOffset_files((int) out.position());
      out.write(files.toByteBuffer());//2
      out.position(out.position() + (512 - out.position() % 512));
      header.setOffset_lists((int) out.position());
      out.write(lists.toByteBuffer()); //3      
      out.position(out.position() + (512 - out.position() % 512));
      header.setOffset_list_entries((int) out.position());
      header.setSearch_list(lists.getSearchListId());
      out.write(listentries.toByteBuffer());//4
      out.position(out.position() + (512 - out.position() % 512));
      header.setOffset_paths((int) out.position());
      out.write(paths.toByteBuffer()); //5
      out.position(out.position() + (512 - out.position() % 512));
      header.setOffset_strings((int) out.position());

      out.write(strings.toByteBuffer()); //6

      out.position(out.position() + (512 - out.position() % 512));
      header.setOffset_private_data((int) out.position());

      //TODO v2.0 see if using the private data has any use.
      //NOT NEEDED no private data available yet
      // out.write(privateData.toByteBuffer());//7

      out.position(0);
      out.write(header.toByteBuffer());//1
      out.position(502);
      out.close();
      outStream.close();
    } catch (IOException e) {
      //TODO v2.0 Make better exceptionhandling
      e.printStackTrace();
    }
  }

  /**
   * 
   */
  private void makeBackup() {
    try {
      File original = new File(root.getCanonicalPath() + "/" + FILE_NAME);
      File backupFile = new File(root.getCanonicalPath() + "/" + FILE_NAME
          + ".bak");
      backupFile.delete();
      original.renameTo(new File(root.getCanonicalPath() + "/" + FILE_NAME
          + ".bak"));
    } catch (IOException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

  }

  /* (non-Javadoc)
   * @see java.lang.Runnable#run()
   */
  public void run() {
    processed = 0;
    addStrings(stringQ);

    fileCount = foundFiles.size();
    //System.out.println("RESULTING FILES");
    //System.out.println(fileCount);
    System.out.println("set filecount to:" + fileCount);
    ArcAudioFileDecorator data;

    Iterator audioFileIterator = foundFiles.iterator();
    System.out.println("got the Iterator");
    while (audioFileIterator.hasNext() && !(cancel)) {
      File nextFile = (File) audioFileIterator.next();
      Logger.global.finest("START: " + nextFile.getAbsolutePath());
      data = new ArcAudioFileDecorator(ArcLibUtils.getDecodedFile(nextFile));
      //Distribute data to all sections
      //this "if" is a dirty workaround for files without ID3information
      if (data.getMetaData() != null) {
        strings.take(data);
        paths.take(data);
        files.take(data);
        lists.take(data);
      }

      ++processed;
      Logger.global.finest("DONE: " + nextFile.getAbsolutePath());
      setPercentage(processed * 100 / fileCount);
    }
    lists.sort(true);
    listentries.take(lists, files);
    //  PrepareObjects for Output
    //lists.add();

    header.setNum_files(files.size());
    header.setNum_lists(lists.size());
    Logger.global.finest("Starting to write the lib.jbm file");
    write();
    Logger.global.finest("Done writing. Everything should be fine.");
    fireStateChanged();
  }

  /**
   * 
   */
  public void cancel() {
    cancel = true;
  }

}